//Dual steppermotor clock with lasers
// hour stepper 1 to 2 steps

// communication with RTC and RTC lib
#include <Wire.h>
#include "RTClib.h"

RTC_DS3231 rtc;

//stepper variables and lib

#include <Stepper.h>
#define STEPS 2038 // the number of steps in one revolution of your motor (28BYJ-48)
Stepper stepper1(STEPS, 9, 11, 10, 12); //minute stepper
Stepper stepper2(STEPS, 3, 7, 6, 8); //hour stepper
int startup = 0; // counter for resetting stepperposition
int Led = 13; // define LED pin
int buttonpin = 2; // define photo interrupter signal pin
int val; //define a numeric variable
int hou;// hour correction to one extra step every second minute

// Startup, RTC check, calibration of steppers, adjusting steppers to current time
void setup()
{
  // RTC Time input
  Serial.begin(115200);
  delay(1000); // wait for console opening

  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }

  if (rtc.lostPower()) {
    Serial.println("RTC lost power, lets set the time!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }

  //Steppermotor calibration
  pinMode(Led, OUTPUT); // LED pin as output
  pinMode(buttonpin, INPUT); //photo interrupter pin as input

  Minstepcali ();

  Houstepcali ();

  // time check with RTC
  DateTime now = rtc.now();

  // adjusting steppers to current time
  stepper1.setSpeed(10); // 10 rpm
  stepper1.step(now.minute() * 18); // adjust minute stepper

  stepper2.setSpeed(10); // 10 rpm
  if (now.hour() >= 12) {
    stepper2.step((now.hour() - 12) * 90 + now.minute()); // adjust hour stepper
  }
  else if (now.hour() <= 12) {
    stepper2.step(now.hour() * 90 + now.minute()); //  adjust hour stepper
  }

  // 66-73 control for testing purpose only
  Serial.print("Time ");
  Serial.print(now.hour());
  Serial.print(":");
  Serial.print(now.minute());
  Serial.print(":");
  Serial.print(now.second());
  Serial.println("");

}

void loop() {

  // update of RTC time
  DateTime now = rtc.now();

  // Reset minute and hour stepper at 12:00:00
  if (now.hour() == 12 && now.minute() == 00 && now.second() == 00) {

    // minute stepper calibration
    Minstepcali ();

    // hour stepper calibration
    Houstepcali ();

    // delay so that event does not repeat
    delay(1000);
  }

  // stepper motor reset at 0:00:00
  if (now.hour() == 00 && now.minute() == 00 && now.second() == 00) {

    // minute stepper calibration
    Minstepcali ();


    // delay so that event does not repeat
    delay(1000);

  }

  // hourly minute stepper reset

  if (now.minute() == 0 && now.second() == 0 ) {

    Minstepcali ();

    // hour stepper calibration
    Houstepcali ();

    stepper2.setSpeed(10); // 10 rpm
    if (now.hour() >= 12) {
      stepper2.step((now.hour() - 12) * 90 + now.minute()); // adjust hour stepper
    }
    else if (now.hour() <= 12) {
      stepper2.step(now.hour() * 90 + now.minute()); //  adjust hour stepper
    }

    // delay so that event does nor repeat
    delay(1000);

  }

  // minute stepper one minute increment, hourstepper one minute increment

  if (now.second() == 59) {

    stepper1.setSpeed(10); // 10 rpm
    stepper1.step(18); // 18 stepps equals one minute

    if (hou == 0) {
      hou = 1;
    }
    else {
      hou = 0;
    }

    stepper2.setSpeed(10); // 10 rpm
    stepper2.step(1 + hou); // 1 stepp equals one minute on hour steppper

    // delay so that event does not repeat
    delay(1000);

    // Turn stepper2 motor off
    //  digitalWrite(3, LOW);
    //    digitalWrite(7, LOW);
    //   digitalWrite(6, LOW);
    //    digitalWrite(8, LOW);

    // Turn stepper1 motor off
    //   digitalWrite(9, LOW);
    //   digitalWrite(11, LOW);
    //  digitalWrite(10, LOW);
    //   digitalWrite(12, LOW);

  }

  // for testing purpose only
  digitalWrite(Led, LOW);

  Serial.print("Time ");
  Serial.print(now.hour());
  Serial.print(":");
  Serial.print(now.minute());
  Serial.print(":");
  Serial.print(now.second());
  Serial.println("");

}

void Minstepcali () { // calibration of stepper 1
  // minute stepper calibration

  // Power on stepper two to avoid involentery movement
  stepper2.setSpeed(10); // 10 rpm

  do {
    val = digitalRead(buttonpin); //read the value of the photointerrupter
    if (val == HIGH) // turn on LED when photointerrupter is blocked
    {
      stepper1.setSpeed(10); // 10 rpm
      stepper1.step(50); // 50 steps from resetpoint
      startup = startup + 1;
      digitalWrite(Led, HIGH);
      Serial.println("PIN minute high");
    }
    else
    {
      stepper1.setSpeed(10); // 5 rpm
      stepper1.step(-1); // stepping towards photointerrupter
      digitalWrite(Led, LOW);
      Serial.println("PIN minutelow");
    }
  } while (startup < 2);

  stepper1.setSpeed(10);
  stepper1.step(452);
  startup = 0;



}

void Houstepcali () {

  // hour stepper calibration
  do {
    val = digitalRead(buttonpin); //read the value of the photointerrupter
    if (val == HIGH) // turn on LED when photointerrupter is blocked
    {
      stepper2.setSpeed(10); // 10 rpm
      stepper2.step(50); // 50 steps from resetpoint
      startup = startup + 1;
      digitalWrite(Led, HIGH);
      Serial.println("PIN hour high");
    }
    else
    {
      stepper2.setSpeed(10); // 5 rpm
      stepper2.step(-1); // stepping towards photointerrupter
      digitalWrite(Led, LOW);
      Serial.println("PIN hour low");
    }
  } while (startup < 2);
  startup = 0;
  digitalWrite(Led, LOW);
  stepper2.setSpeed(10);
  stepper2.step(357);

}
